package bitsUnited;

/*
 $Id: AbsolutFieldObject.java,v 1.15 2002/07/07 11:28:28 dd Exp $
 $Log: AbsolutFieldObject.java,v $
 Revision 1.15  2002/07/07 11:28:28  dd
 fuer den Ball distChange und dirchange eingebaut. in AbsolutBall zusaetzliche Positionsveraenderung
 verfuegbar ueber getPosCh_X() bzw. getPosCh_y()
 CV : ----------------------------------------------------------------------

 Revision 1.14  2002/07/07 10:05:46  dd
 *** empty log message ***

 Revision 1.13  2002/07/05 08:58:12  dd
 Exceptions weitgehend ausgetrieben

 Revision 1.12  2002/07/03 19:38:15  ju
 maxTimeStamp wieder auf 10 gesetzt

 Revision 1.11  2002/07/02 23:47:27  ju
 package von robocop in bitsUnited umbenannt; mehr nicht...

 Revision 1.10  2002/06/30 15:18:21  dd
 Weltmodell jetzt auch von Rechts nach Links, Positionsbestimmung genauer

 Revision 1.9  2002/06/28 19:18:56  dd
 Aufruf gendert, Bye-Msg, kleine nderungen

 Revision 1.8  2002/06/26 07:50:17  dd
 Visualisierung verbessert

 Revision 1.7  2002/06/25 08:21:12  dd
 Fahnen, Weltmodell und Visualisierung gecheckt

 Revision 1.6  2002/06/24 22:36:46  ju
 timestamp fuer predict auf 0 (NULL) gesetzt

 Revision 1.5  2002/06/23 23:54:58  ju
 timestamp fuer predict auf 1

 Revision 1.4  2002/06/20 16:06:42  dd
 Stand nach unserem "Get-together" Treffen

 Revision 1.3  2002/05/31 18:20:09  dd
 Weltmodell implementiert und RelativFieldObjects angepasst.

 Revision 1.2  2002/05/29 18:26:36  dd
 Erste Version

 */

/**
 * The <code>AbsolutFieldObject</code> class is used as a base class for
 * Field-Objects in absolut coordinates.
 * The classes <code>AbsolutBall</code> and <code>AbsolutSpieler</code>
 * are derivated from <code>AbsolutFieldObject</code>
 * The class is updated by the <code>Weltmodell</code> if with a new
 * see-Message the Object is still visible. Otherwise the Object may be
 * "interpolated" according to its last movement as long as the timestamp
 * did not go to far.
 *
 * @author <a href="mailto:dedietri@uni-mannheim.de">Dennis Dietrich</a>
 * @version 0.1
 */
public class AbsolutFieldObject {

    // Konstanten zur Typbestimmung
 
    /** constant value for an abstract Field Object */
    static public final int ABSTRACT = 0;
    /** constant value for the Ball Object */
    static public final int BALL     = 5;
    /** constant value for a player object */
    static public final int SPIELER  = 6;

    // Konstante fuer die Verfallszeit eines Objektes
 
    /** constant value for the validness of an object */
    static private final int MAXTIMESTAMP = 5;

    private float x;  // Absolute Position x-Wert
    private float y;  // Absolute Position y-Wert

    // value representing the difference of the x-value between two timestamps 
    private float diff_x;  // Differenz zum letzten Wert x
    // value representing the difference of the y-value between two timestamps
    private float diff_y;  // Differenz zum letzten Wert y

    private int timestamp;     // Timestamp der letzten Position
    private int lastTimestamp; // Timestamp des letzten Updates

   /**
     * Creates a new <code>AbsolutFieldObject</code> instance.
     *
     * @param x a <code>float</code> value representing the x-value
     * of the current position
     * @param y a <code>float</code> value representing the y-value
     * of the current position
     * @param timestamp a <code>int</code> value representing the
     * timestamp for the new position
     * 
     * @see AbsolutBall
     * @see AbsolutSpieler
     * @see Weltmodell
     * @see VisualWM
     */
    public AbsolutFieldObject(float x, float y, int timestamp ) {
	this.x = x;
	this.y = y;
	this.timestamp = timestamp;
        // Neues Objekt, kein Vorgaengerwert -> Differenz 0
        diff_x = 0;
	diff_y = 0;
    }

    /**
     * Returns the type of <code>AbsolutFieldObject</code>
     * Overwritten by derivates.
     * Returns the value <code>AbsolutFieldObject.ABSTRACT</code>
     * 
     * @see AbsolutBall
     * @see AbsolutSpieler
     */
    public int getType() {
	return ABSTRACT;
    }
    
   /**
     * Updates the <code>AbsolutFieldObject</code> with new data.
     *
     * @param x a <code>float</code> value representing the x-value
     * of the current position
     * @param y a <code>float</code> value representing the y-value
     * of the current position
     * @param timestamp a <code>int</code> value representing the
     * timestamp for the new position
     * 
     * @see AbsolutBall
     * @see AbsolutSpieler
     */
    public void update( float x, float y, int timestamp ) {
	// Es gibt eine neue Position, also Veraenderung zur alten speichern
	if (timestamp != this.timestamp) {
            // die diff's beziehen sich auf einen step 
	    diff_x = x - this.x / ( timestamp - this.timestamp );
	    diff_y = y - this.y / ( timestamp - this.timestamp );
	}
	// Jetzt die neuen Werte uebernehmen
        this.x = x;
	this.y = y;
	this.timestamp = timestamp;
        this.lastTimestamp = timestamp;
    }
   /**
     * Interpolates a new position from the last known movement.
     *
     * @param timestamp a <code>int</code> value representing the
     * timestamp for the next interpolation
     * 
     * @see AbsolutBall
     * @see AbsolutSpieler
     */
    public boolean predict(int timestamp) {
	// Falls weniger als MAXTIMESTAMP Zeit, seit der letzten Sichtung
        // vergangen sind
        // erstmal lineare Weiterfuehrung spaeter vielleicht Interpolation
	if ( (timestamp - this.lastTimestamp) <= MAXTIMESTAMP && ( timestamp > 0 )) {
	    // letzte Bewegung fortfhren
	    x = x + ( timestamp - this.timestamp ) * diff_x;
	    y = y + ( timestamp - this.timestamp ) * diff_y;
	    this.timestamp = timestamp;
	    // prediction wurde ausgefuehrt
	    return true;
	}
	else {
	    // prediction ist verfallen - Objekt kann entfernt werden!
	    return false;
	}
    }

   /**
     * Returns the x-coordinate of the Object.
     * 
     * @see AbsolutBall
     * @see AbsolutSpieler
     */
    public float getX() {
	return x;
    }

   /**
     * Returns the Y-coordinate of the Object.
     * 
     * @see AbsolutBall
     * @see AbsolutSpieler
     */
    public float getY() {
	return y;
    }

   /**
     * Returns the last movement in x-direction of the Object.
     * 
     * @see AbsolutBall
     * @see AbsolutSpieler
     */
    public float getDiffX() {
	return diff_x;
    }

   /**
     * Returns the last movement in y-direction of the Object.
     * 
     * @see AbsolutBall
     * @see AbsolutSpieler
     */    
    public float getDiffY() {
	return diff_y;
    }

   /**
     * Returns the timestamp of the Object in y-direction of the Object.
     * 
     * @see AbsolutBall
     * @see AbsolutSpieler
     */
    public int getTimeStamp() {
	return timestamp;
    }

}




