package bitsUnited;

import java.io.*;
import java.net.*;


/*
 * Revision 1.7  2002/07/02 23:47:27  ju
 * package von robocop in bitsUnited umbenannt; mehr nicht...
 *
 * Revision 1.6  2002/07/02 20:02:14  mj
 * exit bei Fehlermeldung in getMsg()
 *
 * Revision 1.5  2002/06/20 22:52:28  dd
 * Weltmodell funktioniert, Parser gefixt, Client lauffhig
 *
 * Revision 1.4  2002/06/20 16:06:41  dd
 * Stand nach unserem "Get-together" Treffen
 *
 * Revision 1.3  2002/06/18 14:51:28  mj
 * no message
 *
 */

/**
 *
 * This class manages the UDP Connection between the soccer Server and the Client.
 * Each of the 11 players has got his own ServerConnector getting messages and sending responses.
 *
 */
public class ServerConnector {
    // Socket fr ein- und ausgehende Nachrichten
    private DatagramSocket socket;
    // IP-Adresse des Servers
    InetAddress ipAddress;
    // Port-Nummer des Servers
    private int port;
    
    /**
     * Construktor
     *
     * @param serverName: server name or its IP-address connecting to
     * @param port: Port number of the server connecting to
     */
    public ServerConnector(String serverName, int port) {
        try {
            // konstruiere datagram socket und lege es auf einen freien Port
            this.socket        = new DatagramSocket();
            // setze Serveradresse
            this.ipAddress = InetAddress.getByName(serverName);
            // setze Zielport des Servers
            this.port    = port;
        }
        catch (SocketException se) {
            System.err.println("ServerConnector: kein Verbimdungsaufbau moeglich, Programm neu starten");
            System.exit(1);
        }
        catch (UnknownHostException uhe) {
            System.err.println("ServerConnector: Zieladresse nicht bekannt, Programm neu starten");
            System.exit(1);
        }
    } // Ende Konstruktor ServerConnector
    
    /**
     * This method gets a message from the server and returns it as a string
     *
     */
    public String getMsg() {
        // Puffer fuer die Servernachrichten zuweisen
        byte[] buffer = new byte[2048];
        // erstellt ein datagram package
        DatagramPacket packet = new DatagramPacket(buffer, buffer.length);
        try {
            // wartet auf ein gueltiges Paket und nimmt es an
            socket.receive(packet);
        }
        catch (IOException ioe) {
            System.err.println("ServerConnector: Datenpaket nicht empfangen, rcssserver ueberpruefen und Progamm neu starten");
            System.exit(1);
        }
        
        // festlegen des Serverportes fuer die naechsten Aktionen
        port = packet.getPort();
        // Nachricht als String zurueckgeben
        String msg = new String(packet.getData()).substring(0, packet.getLength()-1)+" ";  // schneidet null characters & \n aus dem Buffer
        
        // System.out.println(msg); // Test
        
        // die empfangene Nachricht wird zurueckgegeben
        return msg;
    } // Ende getMsg()
    
    
    /**
     * This method sends a message to the server
     *
     */
    public void sendMsg(String msg) {
        try {
            // konvertiert die Nachricht in ein Byte Array
            byte[] buffer = msg.getBytes();
            // stelle das Paket aus dem Buffer zusammen und haenge den Servernamen und Port an
            DatagramPacket packet =
            new DatagramPacket(buffer, buffer.length, ipAddress, port);
            // sende das Paket zum Server
            socket.send(packet);
            // System.out.println(msg);
        } // Ende try(sendMsg)
        catch (IOException ioe) {
            System.err.println("ServerConnector: Daten nicht verschickt, rcssserver checken");
        } // Ende catch(sendMsg)
    } // Ende sendMsg()
    
} // Ende ServerConnector
