package bitsUnited;

/*
 $Id: Strategy.java,v 1.25 2002/07/07 23:06:34 mj Exp $
 $Log: Strategy.java,v $
 Revision 1.25  2002/07/07 23:06:34  mj
 ...final version... may the force be with u...

 Revision 1.24  2002/07/07 21:18:08  mj
 ...ball preview...
 
 Revision 1.22  2002/07/07 17:08:26  mj
 *** ueberarbeitet ***
 
 Revision 1.19  2002/07/06 22:29:56  ju
 ...feintuning der methoden...
 
 Revision 1.18  2002/07/03 10:54:10  ju
 aenderung bei searchBall
 
 Revision 1.17  2002/07/02 23:47:27  ju
 package von robocop in bitsUnited umbenannt; mehr nicht...
 
 Revision 1.16  2002/07/02 18:17:54  ju
 ...oldpos entfernt...
 
 Revision 1.14  2002/07/02 10:58:28  ju
 no message
 
 Revision 1.12  2002/07/01 21:14:09  ju
 goTo-Methode in Strategy
 
 Revision 1.10  2002/06/27 19:09:31  ju
 neue methoden in Strategy
 
 Revision 1.9  2002/06/21 16:17:31  ju
 init in Strategy, keine Attribute mehr in Strategy und Sturm
 
 Revision 1.7  2002/06/20 16:06:41  dd
 Stand nach unserem "Get-together" Treffen
 
 Revision 1.6  2002/06/12 17:40:17  ju
 Ball an erste Stelle in knownObjects gesetzt
 
 Revision 1.5  2002/06/11 20:49:50  ju
 added the comments and the method makeDecision
 
 Revision 1.3  2002/06/09 12:21:52  dd
 Fahnenumrechnung in RelativFahne eingebaut
 
 Revision 1.2  2002/06/03 14:47:03  ju
 no message
 
 Revision 1.1.1.2  2002/05/31 11:12:44  ju
 no message
 
 */

import java.util.*;
import java.lang.*;

/**
 * the <code>Strategy</code> is the supper-class of the single strategy components.
 * Only the global variables for all the sub-classes are saved here. The empty
 * method <code>makeDecision</code> is being over-written by the sub-classes and
 * calles the actions of the <code>Player</code>.
 */
public abstract class Strategy {
    
     /**
      * reference to the player
      */
    protected Player myPlayer;
    
    /** initializes the player
     */
    public void init() {
        // Wird im Torwart berschrieben!
        this.myPlayer.init();
    }
    
    /*
     * introduces the <code>Player</code> to the <code>Strategy</code>
     */
    public void connect(Player player) {
        this.myPlayer = player;
    }
    
 /**
  * calculates the distance between two objects of <code>Position</code>.
  * @param pos1X x-coordinate of observed object
  * @param pos1Y y-coordinate of observed object
  * @param pos2X x-coordinate of sensing object
  * @param pos2Y y-coordinate of sensing object
  */
    protected float dist(float pos1X, float pos1Y, float pos2X, float pos2Y) {
        // hier ist die Reihenfolge der Objekte egal...
        return (float)Math.sqrt((pos1Y-pos2Y)*(pos1Y-pos2Y)+(pos1X-pos2X)*(pos1X-pos2X));
    }
    
 /**
  * calculates the direction from an objects of <code>Position</code> to another.
  * @param pos1X x-coordinate of observed object
  * @param pos1Y y-coordinate of observed object
  * @param pos2X x-coordinate of sensing object
  * @param pos2Y y-coordinate of sensing object
  */
    protected float dir(float pos1X, float pos1Y, float pos2X, float pos2Y) {
        // Achtung: es wird die Richtung des Objektes pos1 vom Objekt pos2 aus berechnet!!
        if ( pos1X > pos2X ) {
            return (float)Math.toDegrees((float)Math.atan((pos1Y-pos2Y)/(pos1X-pos2X)));
        }
        else {
            if ( pos1Y > pos2Y ) {
                return (180+(float)Math.toDegrees(Math.atan(pos1Y-pos2Y)/(pos1X-pos2X)));
            }
            else {
                return (-180+(float)Math.toDegrees(Math.atan(pos1Y-pos2Y)/(pos1X-pos2X)));
            }
        }
    }
    
 /**
  * turnes the body to a given direction
  * @param dir desired destination angle 
  */
    protected void turnBodyTo(float dir) {
        if (myPlayer.getReceivedSeeBody()) {
            float ba = myPlayer.getBodyAngle();
            if ((dir-ba) > 180) myPlayer.turnBody(-360+dir-ba);
            else
                if ((dir-ba) < -180) myPlayer.turnBody(360+dir-ba);
                else myPlayer.turnBody(dir-ba);
        }
    }
    
 /**
  * turnes the head to a given direction
  * @param dir desired destination angle 
  */
    protected void turnHeadTo(float dir) {
        if (myPlayer.getReceivedSeeHead()) {
            float ha = myPlayer.getHeadAngle();
            float ba = myPlayer.getBodyAngle();
            myPlayer.turnHead(dir-ha-ba);
        }
    }
    
 /**
  * makes the player kick the ball with given power to given direction
  * @param pow power of shot
  * @param dir dircetion of shot
  */
    protected void kickTo(int pow, float dir) {
        float ba = myPlayer.getBodyAngle();
        if ((dir-ba) > 180) myPlayer.kick(pow, -360+dir-ba);
        else
            if ((dir-ba) < -180) myPlayer.kick(pow, 360+dir-ba);
            else myPlayer.kick(pow, dir-ba);
    }
    
 /**
  * lets the player move to a given position with a given precision
  * @param x destination x-coordinate
  * @param y destination y-coordinate
  * @param delta precision tolerance
  */
    protected boolean goTo(float x, float y, float delta) {
        float ba = myPlayer.getBodyAngle();
        if (dist(x, y, myPlayer.getMyPos().getX(), myPlayer.getMyPos().getY())>delta) {
            if ( (-2f<=(dir(x, y, myPlayer.getMyPos().getX(), myPlayer.getMyPos().getY())-ba)) &&
            ((dir(x, y, myPlayer.getMyPos().getX(), myPlayer.getMyPos().getY())-ba)<=2f) ) {
                
                if (dist(x, y, myPlayer.getMyPos().getX(), myPlayer.getMyPos().getY())>5) {
                    myPlayer.dash(100);
                }
                else {
                    myPlayer.dash(70);
                }
                
                return false;
            }
            else { turnBodyTo( dir(x, y, myPlayer.getMyPos().getX(), myPlayer.getMyPos().getY()) ); return false; }
        }
        else {
            return true;
        }
    }
    
 /**
  * makes the player search for the ball
  */
    protected void searchBall() {
        float ba = myPlayer.getBodyAngle();
        float ha = myPlayer.getHeadAngle();
        float ht = myPlayer.getHT();
        if (ha>=90) {
            if (myPlayer.getHT()==1 && myPlayer.getReceivedSeeBody() && myPlayer.getReceivedSeeHead()) {
                myPlayer.turnBody(180);
                myPlayer.turnHead(-180);
                myPlayer.setHT(0);
            }
            else {
                if (myPlayer.getReceivedSeeHead()) {
                    myPlayer.turnHead(-180);
                    myPlayer.setHT(1);
                }
            }
        }
        else {
            myPlayer.turnHead(25);
        }
    }
    
    
 /**
  * makes the player search for the ball without moving the body; is used for searching the ball during movement
  */
    protected void searchBallWalk() {
        if (myPlayer.getReceivedSeeHead()) {
            float ba = myPlayer.getBodyAngle();
            float ha = myPlayer.getHeadAngle();
            if (ha==90) {
                myPlayer.turnHead(-180);
            }
            else {
                myPlayer.turnHead(25);
            }
        }
    }
    
    
 /** empty method; will be over-written by the sub-classes */
    public void makeDecision() {
    }
    
}