package bitsUnited;

import javax.swing.*;
import java.awt.*;
import java.util.*;

/*
  $Id: VisualWM.java,v 1.9 2002/07/07 10:05:46 dd Exp $
  $Log: VisualWM.java,v $
  Revision 1.9  2002/07/07 10:05:46  dd
  *** empty log message ***

  Revision 1.8  2002/07/05 08:58:12  dd
  Exceptions weitgehend ausgetrieben

  Revision 1.7  2002/07/02 23:47:27  ju
  package von robocop in bitsUnited umbenannt; mehr nicht...

  Revision 1.6  2002/07/02 09:56:59  dd
  VisualWM entflackert, kleinere nderungen

  Revision 1.5  2002/06/28 19:18:56  dd
  Aufruf gendert, Bye-Msg, kleine nderungen

  Revision 1.4  2002/06/25 08:21:12  dd
  Fahnen, Weltmodell und Visualisierung gecheckt

  Revision 1.3  2002/06/19 14:06:19  dd
  Irgendwoher kam ein ungueltiges Zeichen ans Ende der Datei...

  Revision 1.2  2002/06/14 18:26:35  dd
  Anzeigen der eigenen Position fr VisualWM hinzugefgt

  Revision 1.1  2002/06/12 18:05:12  dd
  Visualisierung des Weltmodells hinzugefgt

 */

/**
 * Visualisatzion for the <code>Weltmodell</code> class
 */
public class VisualWM extends JComponent {

    /* Da ich hierin mitbekommen muss, wann sich etwas an den Weltmodelldaten ndert, muss das jeweilige 
       Weltmodell das VisualWM-Objekt kennen. Wird eine Art unsauberes Observer-Pattern *G*
    */

    // Breite und Hhe des halben Feldes
    private static final float FIELDX = 60;
    private static final float FIELDY = 40;

    // Die anzuzeigenden Spielfeldobjekte
    private Vector knownObjects;
    // Die Position des Eigenen Spielers
    private Position myPos;

    // Teamname der eigenen Mannschaft
    private String teamname;

    
    // Skalierungsfaktoren zur Positionsfestlegung
    private float scalex;
    private float scaley;

    public VisualWM(String teamname) {
	super();
	this.teamname = teamname;
	setDoubleBuffered(true);
    }

    /**
     * This method is being called by the <code>Weltmodell</code> and hands over
     * @param knownObjects the Vector of known <code>AbsolutFieldObjects</code>
     */
    public void update(Vector knownObjects, Position myPos) {
	this.knownObjects = knownObjects;
	this.myPos = myPos;
	repaint(); // Repaint anfordern
    }

    /**
     * Returns the smallest Dimension of the Canvas
     */
    public Dimension getMinimumSize() {
	return new Dimension(240,160);
    }

    /**
     * Returns a suitable Dimension of the Canvas
     */
    public Dimension getPreferredSize() {
	return new Dimension(480,320);
    }

    // Transformiert vom Robocup ins Canvas-System
    private int tX( float rx ) {
	// Mitte plus rx * scalex
	return (int) ((getWidth() / 2) + rx * scalex );
    }

    private int tY( float ry ) {
	// Mitte plus ry * scaley
	return (int) ((getHeight() / 2) + ry * scaley );
    }

    /**
     * Paints the Field and all the known Objects on it.
     * @param g the graphics context of the canvas
     */
    public void paint( Graphics g ) {
	try {
	    scalex = getWidth() / (FIELDX*2);
	    scaley = getHeight() / (FIELDY*2);

	    paintField(g); // Das Spielfeld zeichnen
	    paintMe(g,myPos.getX(),myPos.getY());
	    // Den Vector durchlaufen
	    if ( knownObjects != null ) {
		for ( Enumeration aenum = knownObjects.elements(); 
		      aenum.hasMoreElements(); ) {
		    AbsolutFieldObject temp = (AbsolutFieldObject)aenum.nextElement();
		    switch (temp.getType() ) {
		    case AbsolutFieldObject.BALL:
			AbsolutBall ball = (AbsolutBall)temp;
			paintBall(g, ball.getX(), ball.getY());
			break;
		    case AbsolutFieldObject.SPIELER:
			AbsolutSpieler spieler = (AbsolutSpieler)temp;
			paintPlayer( g, spieler.getX(), spieler.getY(), (teamname.equals(spieler.getTeamName()) ), spieler.getSpielerNum());
		    
		    }
		}
	    }
	}
	catch( Exception e ) {
	    // Entweder eigene Position fehlerhaft oder knownObjects leer
	}
    }

    private void paintField( Graphics g ) {
	// Spielfeldgeometrie knnte man noch in Konstanten auslagern
	g.setColor(Color.green.darker());
	g.fillRect(0,0,getSize().width, getSize().height);
	g.setColor(Color.white);
	g.drawRect( tX(-52.5f),tY(-34), (int)(2*52.5*scalex), (int)(2*34*scaley) );
	g.drawLine( tX(0),tY(-34), tX(0), tY(34));
	float mkdurchmesserx = 2*10*scalex;
        float mkdurchmessery = 2*10*scaley;
	g.drawOval( tX(0)-(int)(0.5*mkdurchmesserx), tY(0)-(int)(0.5*mkdurchmessery), (int)mkdurchmesserx, (int)mkdurchmessery );
	// Die Torksten zeichnen
	g.setColor( Color.black );
	g.fillRect( tX(-52.5f - 2),tY(-7), (int)(2*scalex), (int)(2*7*scaley) );
	g.fillRect( tX(52.5f), tY(-7), (int)(2*scalex), (int)(2*7*scaley) );
    }
    
    private void paintMe( Graphics g, float x, float y ) {
  	g.setColor(Color.yellow);
	g.fillOval( (int)(tX(x)- scalex), (int)(tY(y) - scaley), (int)(2*scalex), (int)(2*scaley));
    }

    private void paintBall( Graphics g, float x, float y ) {
	g.setColor(Color.lightGray);
	g.fillOval( (int)(tX(x)- scalex), (int)(tY(y) - scaley), (int)(2*scalex), (int)(2*scaley)); 
    }

    private void paintPlayer( Graphics g, float x, float y, boolean ownTeam, int number) {
	g.setColor(ownTeam?Color.blue:Color.red);
	g.fillOval( (int)(tX(x)- scalex), (int)(tY(y) - scaley), (int)(2*scalex), (int)(2*scaley));
	g.setColor( Color.white );
	g.drawString( "" + number, (int)(tX(x) - 10), (int)(tY(y) - 10));
    }

}


