package bitsUnited;

/*
  $Id: Weltmodell.java,v 1.19 2002/07/07 15:03:37 dd Exp $
  $Log: Weltmodell.java,v $
  Revision 1.19  2002/07/07 15:03:37  dd
  Position Change Daten werden verarbeitet

  Revision 1.18  2002/07/07 11:28:28  dd
  fuer den Ball distChange und dirchange eingebaut. in AbsolutBall zusaetzliche Positionsveraenderung
  verfuegbar ueber getPosCh_X() bzw. getPosCh_y()
  CV : ----------------------------------------------------------------------

  Revision 1.17  2002/07/07 10:05:46  dd
  *** empty log message ***

  Revision 1.16  2002/07/05 20:23:37  dd
  kleine nderungen, Verbesserung beim Winkel im Weltmodell

  Revision 1.15  2002/07/02 23:47:27  ju
  package von robocop in bitsUnited umbenannt; mehr nicht...

  Revision 1.14  2002/07/02 09:56:59  dd
  VisualWM entflackert, kleinere nderungen

  Revision 1.13  2002/06/30 15:18:21  dd
  Weltmodell jetzt auch von Rechts nach Links, Positionsbestimmung genauer

  Revision 1.12  2002/06/26 07:50:17  dd
  Visualisierung verbessert

  Revision 1.11  2002/06/23 23:58:18  ju
  no message

  Revision 1.10  2002/06/23 14:34:13  mj
  Achtung: neue Methode hinzugefuegt, die BodyAngle (hier genannt Blickwinkel)
  verfuegbar macht

  Revision 1.9  2002/06/21 08:40:56  dd
  init in Strategie
 
  Revision 1.8  2002/06/20 22:52:28  dd
  Weltmodell funktioniert, Parser gefixt, Client lauffhig
 
  Revision 1.7  2002/06/19 14:07:21  dd
  Gleiches Problem wie bei VisualWM ausserdem war das anscheinend eine aeltere Version des Weltmodells
 
  Revision 1.6  2002/06/14 18:26:35  dd
  Anzeigen der eigenen Position fr VisualWM hinzugefgt
 
  Revision 1.5  2002/06/12 18:02:52  dd
  Visualisierung fr das Weltmodell neu und Weltmodell erweitert
 
  Revision 1.4  2002/06/12 17:40:17  ju
  Ball an erste Stelle in knownObjects gesetzt
 
  Revision 1.3  2002/06/09 12:21:52  dd
  Fahnenumrechnung in RelativFahne eingebaut
 
   Revision 1.2  2002/05/31 18:20:09  dd
   Weltmodell implementiert und RelativFieldObjects angepasst.
 
   Revision 1.1  2002/05/29 18:28:34  dd
   Erste Version
 */

import java.util.*;

/**
 * The <code>Weltmodell</code> class is used to determine from the given
 * <code>RelativFieldObject</code>s the absolut position and view-angle
 * of the Player and then determines the position of all the other
 * visible or still known objects.
 *
 */
public class Weltmodell{
    
    /*
      Zur Koordinatenfestlegung:
      Spielfeldmittelpunkt ist (0,0).
      Blickt der Spieler an (0,0) Richtung gegnerisches Tor, so sind die
      Koordinatenwerte hinter ihm (eigene Hlfte) negativ und die vor ihm
      ( gegnerische Hlfte) positiv.
      Alles links von ihm ist negativ, alles rechts positiv.
     
                                 ^
                          --     |   +-
            |
                      -----------+----------> Gegnertor
                                 |
                          -+     |   ++
                                 |
     
     Drehungen sind im Uhrzeigersinn positiv!
     */
    
    // Umrechnungsfaktor von Gradma nach Bogenma
    private static final double DEG2BM = (2* Math.PI) / 360;
    
    // Spielen wir von Links nach Rechts?
    private boolean left2right;

    // Der Name des eigenen Teams - alles andere sind Gegner!
    private String teamName;
    
    // Der errechnete Blickwinkel
    private float blickwinkel;
    
    // Die errechnete Position des eigenen Spielers
    private Position myPos;
    
    // Die dem Weltmodell bekannten AbsolutFieldObjects
    private Vector knownObjects;
    
    // Die Visualiserungskomponente - null, falls an dieses Weltmodell keine angebunden ist
    private VisualWM visWM;
    
    /**
     * Constructor for the <code>Weltmodell</code>
     * @param left2right is true if the player plays from left to right
     * @param teamName the player's teamName for identifying other players
     */
    public Weltmodell( boolean left2right ) {
        this.left2right = left2right;
        this.knownObjects = new Vector();
    }
    
    /**
     * is called by the Player in order to update the player's view-angle,
     * position and the <code>knownObjects</code>-Vector.
     * @param relativObjects is a Vector of <code>RelativFieldObjects</code>
     * @param oldblickwinkel the previously computed angle of sight from the <code>Player</code> object
     * as delivered by the parser module.
     * @see RelativFieldObject
     */
    public void update(Vector relativObjects, float oldblickwinkel) {
        
        // Absoluten Blickwinkel bestimmen
        // Prmisse: Es wird immer wenigstens eine Linie wahrgenommen
        blickwinkel = compBlickwinkel(relativObjects, oldblickwinkel);
        
        // Absolute Position bestimmen
        myPos = compMyPos(relativObjects);
        
        // RelativObjekte traversieren und in AbsolutObjekte transformieren
        // dabei nicht mehr aktualisierte Objekte predicten oder aus knownObj
        // entfernen.
        //  **** ACHTUNG! Entfernt Eintrge aus relativObjects! *****
        updateKnownObjects(relativObjects);
    }
    
    private float compBlickwinkel( Vector relativObjects, float oldblickwinkel ) {
        // Absoluten Blickwinkel bestimmen
        // Prmisse: Es wird immer wenigstens eine Linie wahrgenommen
        
        float awinkel = oldblickwinkel;  // falls nicht, den alten blickwinkel bernehmen
        RelativLinie linie = null;
        
        // Nach der am weitesten entfernten Linie suchen - damit die Bestimmung auch auerhalb der Auenlinien klappt. ?!
        for ( Enumeration renum = relativObjects.elements();
        renum.hasMoreElements(); ) {
            // Cast unbedenklich, weil Vector of RelativFieldObjects
            RelativFieldObject temp = (RelativFieldObject) renum.nextElement();
            
            // Ist das RelativObject eine Linie?
            if ( temp.getType() == RelativFieldObject.LINIE ) {
                // Dann kann ich nach RelativLinie casten!
                if ( linie == null ) {
                    linie = (RelativLinie)temp;
                }
                else {
                    if ( linie.getDistance() < temp.getDistance() ) {
                        linie = (RelativLinie) temp;
                    }
                }
            }
        }
        if ( linie != null ) {
            float rwinkel = linie.getAngle();
            
            // Blickwinkelbestimmung
            // ACHTUNG!! Unsicher ob der Spieler sich auch auerhalb der Auenlinien korrekt orientiert!
            if ( left2right ) {
                // Spiel von Links nach Rechts
                switch ( linie.getLineChar() ) {
                    case 'b':
                        awinkel = ((rwinkel >= 0)?180:0) - rwinkel;
                        break;
                    case 't':
                        awinkel = ((rwinkel >= 0)?0:-180) - rwinkel;
                        break;
                    case 'l':
                        awinkel = ((rwinkel >= 0)?-90:90) - rwinkel;
                        break;
                    case 'r':
                        awinkel = ((rwinkel >= 0)?90:-90) - rwinkel;
                        break;
                }
            }
            else {
                // Spiel von Rechts nach Links
                switch ( linie.getLineChar() ) {
                    case 'b':
                        awinkel = ((rwinkel >= 0)?0:-180) - rwinkel;
                        break;
                    case 't':
                        awinkel = ((rwinkel >= 0)?180:0) - rwinkel;
                        break;
                    case 'l':
                        awinkel = ((rwinkel >= 0)?90:-90) - rwinkel;
                        break;
                    case 'r':
                        awinkel = ((rwinkel >= 0)?-90:90) - rwinkel;
                        break;
                }
            }
        }
        
        // System.out.println("Blickwinkel: " + awinkel );
        return awinkel;
    }
    
    private Position compMyPos( Vector relativObjects ) {
        // Absolute Eigenposition bestimmen
        // Erster Ansatz: Absolute Position ausschlielich an der naechstge-
        // legensten Fahne festmachen
	// Verbesserung: Alle Fahnen via gewichtetem Mittel einbeziehen
        
	float    distance_sum = 0;
	float     xsum = 0;
	float     ysum = 0;

	for ( Enumeration renum = relativObjects.elements(); renum.hasMoreElements(); ) {
	    RelativFieldObject temp = (RelativFieldObject) renum.nextElement();
	    if ( temp.getType() == RelativFieldObject.FAHNE ) {
		RelativFahne fahne = (RelativFahne) temp;
		float w_abs = blickwinkel + fahne.getAngle();

		float deltax = fahne.getDistance() * (float)Math.cos(DEG2BM * w_abs);
		float deltay = fahne.getDistance() * (float)Math.sin(DEG2BM * w_abs);

		xsum += (fahne.getX(left2right) - deltax) / fahne.getDistance();
		ysum += (fahne.getY(left2right) - deltay) / fahne.getDistance();

		distance_sum += 1 / fahne.getDistance();
	    }
	}

	if ( distance_sum > 0 ) 
	    return new Position( xsum / distance_sum, ysum / distance_sum );
	else
	    return null;
    }
    
    private void updateKnownObjects(Vector relativObjects) {
        // zunchst den Vector der knownObjects durchgehen und prfen ob es ein passendes
        // RelativFieldObject gibt. Falls ja, damit das bereits bekannt updaten,
        // falls nein, das AbsolutFieldObject predicten oder falls dies nicht mehr
        // machbar ist, das AbsolutObject lschen.
        // Nach dem Update wird das RelativFieldObject aus dem Vector entfernt.
	if ( (knownObjects != null) && (myPos != null)) {
	    for ( Enumeration aenum = knownObjects.elements();
		  aenum.hasMoreElements(); ) {
		AbsolutFieldObject atemp = (AbsolutFieldObject) aenum.nextElement();
		switch (atemp.getType()) {
                case AbsolutFieldObject.BALL:
		    {
			AbsolutBall  aball = (AbsolutBall) atemp;
			int timestamp = aball.getTimeStamp();  // evtl. zum predicten notwendig
			boolean isUpdated = false;
			for ( Enumeration renum = relativObjects.elements();
			      renum.hasMoreElements(); ) {
			    RelativFieldObject rtemp = (RelativFieldObject) renum.nextElement();
			    timestamp = rtemp.getTimeStamp(); // Timestamp auf den aktuellen Stand setzen
			    if ( rtemp.getType() == RelativFieldObject.BALL ) {
				RelativBall rball = (RelativBall) rtemp;
				float w_abs = blickwinkel + rball.getAngle();
				float x = myPos.getX() + (rball.getDistance() * (float) Math.cos(DEG2BM * w_abs));
				float y = myPos.getY() + (rball.getDistance() * (float) Math.sin(DEG2BM * w_abs));
				aball.update(x,y,rball.getTimeStamp());
				// Jetzt noch die Positionsaenderung in X und Y Richtung bestimmen.
				// Dazu die alte Position errechnen. Die Differenz ist der Verschiebungsvector
				float w_abs_alt = blickwinkel + rball.getAngle() - rball.getDirChange();
				float x_alt = myPos.getX() + ((rball.getDistance() - rball.getDistChange()) * (float) Math.cos(DEG2BM * w_abs_alt));
				float y_alt = myPos.getY() + ((rball.getDistance() - rball.getDistChange()) * (float) Math.sin(DEG2BM * w_abs_alt));
				aball.setPosCh_X( x - x_alt );
				aball.setPosCh_Y( y - y_alt );
                            // aus relativObjects lschen - gibt einen boolean zurck!
				relativObjects.remove(rball);
				isUpdated = true;
				break;
			    }
			}
			if ( !isUpdated ) {
			    // Predicten, falls zu alt dann lschen
			    if ( !aball.predict( timestamp ) ) {
				knownObjects.remove(aball);
			    }
			}
		    }
		    break;
                case AbsolutFieldObject.SPIELER:
		    {
			AbsolutSpieler aspieler = (AbsolutSpieler) atemp;
			int timestamp = aspieler.getTimeStamp();
			boolean isUpdated = false;
			for ( Enumeration renum = relativObjects.elements();
			      renum.hasMoreElements(); ) {
			    RelativFieldObject rtemp = (RelativFieldObject) renum.nextElement();
			    timestamp = rtemp.getTimeStamp(); // Timestamp auf den aktuellen Stand setzen
			    if (rtemp.getType() == RelativFieldObject.SPIELER ) {
				RelativSpieler rspieler = (RelativSpieler) rtemp;
				if ( (rspieler.getSpielerNum() == aspieler.getSpielerNum()) &&
				     (rspieler.getTeamName() == aspieler.getTeamName()) ) {
				    float w_abs = blickwinkel + rspieler.getAngle();
				    float x = myPos.getX() + (rspieler.getDistance() * (float) Math.cos(DEG2BM * w_abs));
				    float y = myPos.getY() + (rspieler.getDistance() * (float) Math.sin(DEG2BM * w_abs));
				    aspieler.update(x,y,rspieler.getTimeStamp());
				    // aus relativObjects lschen - gibt einen boolean zurck!
				    relativObjects.remove(rspieler);
				    isUpdated = true;
				}
			    }
			}
			if ( !isUpdated ) {
			    if ( !aspieler.predict( timestamp ) ) {
                            knownObjects.remove(aspieler);
			    }
			}
		    }
		    break;
                default:
                    // Es interessieren nur Ball und Spieler im Absoluten
                    break;
		}
	    }
	}
	    
	// jetzt sind noch die Fahnen, Tore, Linien und noch unbekannte Spieler sowie der Ball, falls er vorher
        // noch nicht bekannt gewesen sein sollte, im Vector.
        // Alle noch brigen Spieler und evtl. den Ball in die knownObjects "transplantieren"
	if ( myPos != null ) { 
	    for ( Enumeration renum= relativObjects.elements();
		  renum.hasMoreElements(); ) {
		RelativFieldObject rtemp = (RelativFieldObject) renum.nextElement();
		switch ( rtemp.getType() ) {
                case RelativFieldObject.BALL:
		    {
			RelativBall rball = (RelativBall) rtemp;
			float w_abs = blickwinkel + rball.getAngle();
			float x = myPos.getX() + (rball.getDistance() * (float) Math.cos(DEG2BM * w_abs));
			float y = myPos.getY() + (rball.getDistance() * (float) Math.sin(DEG2BM * w_abs));
			// Jetzt noch die Positionsaenderung in X und Y Richtung bestimmen.
			// Dazu die alte Position errechnen. Die Differenz ist der Verschiebungsvector
			float w_abs_alt = blickwinkel + rball.getAngle() - rball.getDirChange();
			float x_alt = myPos.getX() + ((rball.getDistance() - rball.getDistChange()) * (float) Math.cos(DEG2BM * w_abs_alt));
			float y_alt = myPos.getY() + ((rball.getDistance() - rball.getDistChange()) * (float) Math.sin(DEG2BM * w_abs_alt));
			AbsolutBall aball = new AbsolutBall(x,y,rball.getTimeStamp());
			aball.setPosCh_X( x - x_alt );
			aball.setPosCh_Y( y - y_alt );
			// und schliesslich zu den bekannten Objekten hinzufuegen
			knownObjects.insertElementAt(aball, 0);
                    break;
		    }
                case RelativFieldObject.SPIELER:
		    {
			RelativSpieler rspieler = (RelativSpieler) rtemp;
			// Spieler wird nur bernommen, falls seine Informationen vollstndig sind!
			if ( rspieler.getSpielerNum() != 0 ) {
			    float w_abs = blickwinkel + rspieler.getAngle();
			float x = myPos.getX() + (rspieler.getDistance() * (float) Math.cos(DEG2BM * w_abs));
			float y = myPos.getY() + (rspieler.getDistance() * (float) Math.sin(DEG2BM * w_abs));
			knownObjects.addElement( new AbsolutSpieler( x,y, rspieler.getTimeStamp(),
								     rspieler.getSpielerNum(), rspieler.getTeamName() ) );
			}
			break;
		    }
		}
	    }
	}
        if ( visWM != null ) visWM.update(knownObjects, myPos);  // Die Visualisierung updaten
    }
    
    public Vector getKnownObjects(){
        return knownObjects;
    }
    
    /**
     * Returns the current Position of the Player
     */
    public Position getMyPos() {
        return myPos;
    }

    /**
     *Returns the current angle of the player
     */
    public float getMyAngle() {
        return blickwinkel;
    }
    
    /**
     * Connects the <code>Weltmodell</code> object with a <code>VisualWM</code> object.
     * to diconnect any visualisation call connectVisWM(null)
     * @param visWM the <codeVisualWM</code> Object
     */
    public void connectVisWM( VisualWM visWM ) {
        this.visWM = visWM;
        if ( visWM != null ) visWM.update(knownObjects, myPos);
    }
}

